#include "Delay_Sub.h"
#include "Speakjet_Serial.h"
#include "phonemes.h"
#include "phrases.h"
#include "sLCD.h"
#include "x10.h"
#include "VRbot_Protocol.h"
#include "VRbot.h"
//***************************************************************************
//  HALwSpeech.c
//  PIC16F688 MCU @ 8Mhz Internal Clock, No Crystal Required.
//  Programmed for RCI RGB688V5.pcb motherboard.
//  Uses PIC to send/receive serial commands from VRBot Module.  Pic also
//  sends x10 Bit Commands via PORTA to X10 PL-513 Power Line Interface.
//  VRBot Module programmed using VRbot GUI Program.  See www.veear.eu
//  for further information and manuals.
//  All Voice Commands Programmed using GUI as shown in
//  WordSD0[], WordSD1[], WordSD2[].
//
//  RC0 is Scanning LED Output
//  RC2 is Speaking Input
//  RC3 is Reset Output
//  RA2 is Speech Tx Output
//  RA0 is Zero Crossing Input - Pin 1 PL513,  Pull-up
//  RA1 is Transmit to X10 Output = Pin 4 PL-513,  Pin 2 and 3 GND.
//  RC4 Rx, RC5 Tx to VRBot,  PULL-up.
//  sLCD on PORTA, RA4 and RA5.

//***************************************************************************
char dummy, chan, OnOff, mode;
signed char res, res1;
char House;
char DimVal = 0;
char txt[4];
//                           1234567890123456
const char Message1[17]   = "HAL the Butler  ";
const char Message2[17]   = "P16F688   by RCI";
const char Message3[17]   = "HAL is Activated";
const char Message4[17]   = "Scanning...";
const char Message5[17]   = "Transmitting X10";
const char Message6[17]   = "Program Mode    ";
const char Message12[17]  = "HAL is Ready!   ";
const char Message99[17]  = "                ";
const char HelloDave[] = {22, 85, 158, 187, 1, 22, 70, 174, 7, 154, 166, 166, 0};
//****************************************************************************
//*** Enter Programmed SI and SD Words Here***********************************
//****************************************************************************
const char *WordSD0[] =  {"HAL             ",    // 0
                          "Robot           ",    // 1
                          "                "};   // 2
                          
const char *WordSD1[] =  {"Living Room     ",    // 20
                          "Piano Room      ",    // 21
                          "Kitchen         ",    // 22
                          "Porch           ",    // 23
                          "Bedroom         ",    // 24
                          "Fan             ",    // 25
                          "Bathroom        ",    // 26
                          "Hallway         ",    // 27
                          "Dining Room     ",    // 28
                          "Coffee Pot      ",    // 29
                          "Christmas Tree  ",    // 30
                          "Pond Lights     ",    // 31
                          "                "};   // 32
                          
const char *WordSD2[] =  {"ON              ",    // 0
                          "OFF             ",    // 1
                          "Dim Lights      ",    // 2
                          "                "};   // 3
                          

//****************************************************************************
void Get_SD_Commands(void)
{
    char group = 0;
    while (VRbot_GetGroupCount(group) == 0)
    {
    }
wait_sd:

   VRbot_RecognizeSD(group);          // Check Speaker Dependant Table Group 0
   BytetoStr(group, txt);             // Display Group Value
   sLCD_Outn(2, 14, txt);
//*** Loop Here to Scan for Words within a Group *******
while (1 == 1)
{
   res = VRbot_CheckResult();         // Get Response from VRBOT
   sLCD_Out(2, 1, Message4);          // Display "Scanning"
   if ((res < -1) && (group > 0))     // If command not understood
      {
      Speakjet_Transmit(_sorrydave);
      group = 0;                      // Go Wait for another Group 0 command
      goto wait_sd;
      }
   if (res < -1)                      // If no response
      goto wait_sd;
//******************************************************
// If Response to "HAL" is Valid,  Display Request for Command
   if(group == 0)                     // If VRBot Group 0
    {
    if (res >= 0)                     // If VRbot received valid command
      {
      sLCD_Out(2, 1, WordSD0[res]);   // Display "HAL"
      Speakjet_Transmit(_yesdave);    // Acknowledge
      Delay1000ms();
      sLCD_Out(2, 1, Message99);
      group = 1;                      // Set Group for next command
      goto wait_sd;
      }
    } // End If Group
//******************************************************
// If Room Response is Valid,  Display Recalled Room Data
    if(group == 1)                    // If VRBot Group 1 valid command
    {
      if (res >= 0)
      {
      if (res == 0)                   // Living Room
         {
         chan = 2;                    // x10 Channel Number
         }
      if (res == 1)                   // Piano Room
         {
         chan = 10;                    // x10 Channel Number
         }
      if (res == 2)                   // Kitchen
         {
         chan = 4;                    // x10 Channel Number
         }
      if (res == 3)                   // Porch
         {
         chan = 9;                    // x10 Channel Number
         }
      if (res == 4)                   // Bedroom
         {
         chan = 5;                    // x10 Channel Number
         }
      if (res == 5)                   // Fan
         {
         chan = 8;                    // x10 Channel Number
         }
      if (res == 6)                   // Bathroom
         {
         chan = 6;                    // x10 Channel Number
         }
      if (res == 7)                   // Hallway
         {
         chan = 7;                    // x10 Channel Number
         }
      if (res == 8)                   // Dining Room
         {
         chan = 14;                   // x10 Channel Number
         }
      if (res == 9)                   // Coffee Pot
         {
         chan = 15;                   // x10 Channel Number
         }
      if (res == 10)                  // Christmas Tree
         {
         chan = 3;                    // x10 Channel Number
         }
      if (res == 11)                  // Pond Lights
         {
         chan = 11;                   // x10 Channel Number
         }
      sLCD_Out(2, 1, WordSD1[res]);   // Display Room Selected
      Speakjet_Transmit(_yesdave);    // Acknowledge
      Delay1000ms();
      sLCD_Out(2, 1, Message99);
      group = 2;                      // Set Group for next command
      goto wait_sd;
      }
    } // End If Group
//******************************************************
// If ON/OFF Command Response is Valid,  Display Recall Light Value
    if(group == 2)                   // If VRBot Group 2 valid command
    {
    if (res >= 0)
      {
      if (res == 0)                  // If "ON"
         {
         OnOff = 20;
         }
      if (res == 1)                  // If "OFF"
         {
         OnOff = 28;
         }
      sLCD_Out(2, 1, WordSD2[res]);  // Display "ON" or "OFF"
      Speakjet_Transmit(_yesdave);   // Acknowledge
      Delay1000ms();
      sLCD_Out(2, 1, Message99);
      group = 0;                     // Reset group for another round
      mode = 3;                      // Set mode for x10 transmit
      goto trans;
      }
    } // End If Group
  LED = 1;                    // While waiting for valid command.....
  Delay100ms();               // Pulse LED to show HAL is active and scanning
  LED = 0;
  Delay100ms();
  } // End While
trans:
      return;
} // End Demo_SD
//****************************************************************************
void x10Trans(char House, char Chan, char OnOff, char DimVal)
{
      x10_send_data(House, Chan);  // Transmit House and Channel twice
      x10_send_data(House, Chan);
      Delay500ms();
      x10_send_data(House, OnOff); // Transmit House and Command twice
      x10_send_data(House, OnOff);
}
//****************************************************************************
void main(void)
{
          //  76543210
  ANSEL =   0b00000000;           // All Digitals
  PORTA =   0b00000000;           // Clear All PORTA
  PORTC =   0b00000000;           // Clear All PORTC
  CMCON0 =  0b00000111;           // Set all Comparators Off
  OSCCON =  0b01110001;           // Set Osc @ 8 Mhz
  Uart1_Init(9600);               // VRbot Default 9600 Baud
  Soft_Uart_Init(&PORTA, 7, 2, 9600, 0);  // Speakjet Default 9600 Baud
  TRISA =   0b00001001;           // RA0 = In, RA1 = Out, RA2-RA4-RA5 = Out
  TRISC =   0b00100010;           // RA1 and RA5 are Inputs
  mode = 1;                       // Preset "switch"
  House = 'A';                    // Default Housecode = 'A'
  Init_sDisplay();                // Initialize LCD
  Delay100ms();
  sResetLCD();                    // Reset LCD
  Delay100ms();
  Speakjet_Reset();               // Reset Speakjet Controller
  sLCD_Out(1, 1, Message1);       // Display Hal the Butler
  sLCD_Out(2, 1, Message2);       // Display P16F688   by RCI
  Delay2000ms();
  Speakjet_Transmit(_morningdave); // Welcome Message
  Delay2000ms();
  // Wait here to Detect VRBot Board
  // goto start;  //  Use this if no VRbot connected
loop:
  sLCD_Out(2, 1, Message6);       // Program Mode
  if (VRbot_Detect())             // If VRbot Communicates with PIC...
     {
     sLCD_Out(1, 1, Message3);    // Display Hal is Activated
     sLCD_Out(2, 1, Message99);
     Delay2000ms();
     goto start;
     }
  Delay500ms();
goto loop;                        // Loop back if Vrbot in Prog Mode
start:
//****************************************************************************
while (1 == 1)
  {

switch(mode)
  {
//***************************************************************************
case 1:
      sLCD_Out(1, 1, Message12);        // Display "HAL is Ready!"
      sLCD_Out(2, 1, Message99);
      Delay1000ms();
      Get_SD_Commands();                // Test for Speech Recognition...
      break;
//***************************************************************************
case 2:

      // spare

break;
//***************************************************************************
case 3:  // Transmit Bit Data to X10 PL-513 Interface

      sLCD_Out(1, 1, Message99);
      sLCD_Out(2, 1, Message5);         // Transmitting.....
      x10Trans(House, chan, OnOff, 0);
      Delay100ms();
      DimVal = 0;                       // Clear variables
      OnOff = 0;
      chan = 0;
      mode = 1;                         // Return to Wait for Command

break;
//***************************************************************************

    } // End Switch
  } // End While
} // End Main
//****************************************************************************
//****************************************************************************