//***************************************************************************
//  x10.h
//  PSC05/PL-513 x10 Interface Driver
//  NOTE:  x10 Read Not used for PL-513
//  Connect:
//           P16F688          PL-513     Phone Cable
//              C0              1          Yellow
//              C1              4          Black
//              N/C             3-- GND    Red
//              GND             2-- GND    Green
//
//           Other MCU        PSC05      Phone Cable
//              C0              1          Yellow
//              C1              4          Black
//              C2              3          Red
//              GND             2          Green
//***************************************************************************
//***************************************************************************
#define Zero_Crossing   PORTA.F0   // Receive Zero Crossing from Power Line
#define TRANS_TO_X10    PORTA.F1   // Transmit to X10
#define RCV_FROM_X10    PORTA.F2   // Receive from X10. Not Used for PL-513

char const House_Code_Table[17] = {'M','N','O','P','C','D','A','B','E',
                                  'F','G','H','K','L','I','J'};
char const Unit_Code_Table[18] = {13, 14, 15, 16, 3, 4, 1, 2, 5, 6, 7,
                                  8, 11, 12, 9, 10};
char txt4[4], temp, tempkeypad, setbit;
char house_code, unit_code;
//***************************************************************************
char shiftRight(char number)
{
  return number >>= 1;
}
//***************************************************************************
char x10_data_ready()    // Checks for Data Present from Power Line Interface
{
  return (!RCV_FROM_X10);
}
//***************************************************************************
void x10_zero_cross()
{
  if(Zero_Crossing)            // returns state of input pin
     while(Zero_Crossing) ;
  else
     while(!Zero_Crossing) ;
}
//***************************************************************************
void x10_trans_bits(char bit_val, char numofbits, char startbit)
{
   char loopcount;

   for(loopcount = 1; loopcount <= numofbits; ++loopcount)
     {
     x10_zero_cross();
     bit_val = shiftRight(bit_val);
     TRANS_TO_X10 = bit_val.F0;
     delay_ms(1);
     TRANS_TO_X10 = 0;
     if(startbit == 0)
        {
        x10_zero_cross();
        TRANS_TO_X10 = !bit_val.F0;
        delay_ms(1);
        TRANS_TO_X10 = 0;
        }
     }
     return;
}
//***************************************************************************
void x10_send_data(char house_code2, char unit_code2)
{
  char i;
  i = 0;
  while (House_Code_Table[i] != house_code2)
    i++;
  house_code2 = i;
  if(unit_code2 <= 16)
     {
     i = 0;
     while (Unit_Code_Table[i] != unit_code2)
       i++;
     unit_code2 = i;
     }
  x10_trans_bits(7 * 2, 4, 1);
  x10_trans_bits(house_code2 * 2, 4, 0);
  x10_trans_bits(unit_code2 * 2, 5, 0);
  x10_trans_bits(0, 6, 1);
  return;
}
//***************************************************************************
char x10_read_bits(char num_of_bits)
{
char ic, dataval;

   for(ic = 1; ic <= num_of_bits; ++ic)
      {
      x10_zero_cross();
      delay_us(710);
      if (RCV_FROM_X10 == 1)
         {
         setbit = 128;
         }
      else
         {
         setbit = 0;
         }
      dataval = dataval + setbit;
      dataval = shiftRight(dataval);
      x10_zero_cross();
      delay_us(710);
   }
   for(ic = 0; ic < 7 - num_of_bits; ++ic)
      {
      dataval = shiftRight(dataval);
      }
   return(dataval);
}
//***************************************************************************
void x10_read_pulse_code()   //(char house_code, char key_code)
{
  x10_read_bits(2);
  house_code = x10_read_bits(4);
  house_code = House_Code_Table[house_code];
  unit_code   = x10_read_bits(5);
  if(unit_code < 16)
    unit_code = Unit_Code_Table[unit_code];
}
//***************************************************************************